package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.distributed.print.business.bean.result.analyze.GradeInitResult;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.mapper.GradeInitializeMapper;
import com.qmth.distributed.print.business.service.GradeInitializeService;
import com.qmth.distributed.print.business.service.TBSyncTaskService;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.GradeAnalyzePaperStatusEnum;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.enums.TaskResultEnum;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Description: 教研分析对接-数据初始化服务实现类
 * @Author: CaoZixuan
 * @Date: 2022-05-26
 */
@Service
public class GradeInitializeServiceImpl implements GradeInitializeService {
    @Resource
    GradeInitializeMapper gradeInitializeMapper;
    @Resource
    TBSyncTaskService tbSyncTaskService;


    @Override
    public IPage<GradeInitResult> findNeedAnalyzedPaper(Long semesterId, Long examId, String courseCode, SysUser requestUser, int pageNumber, int pageSize) {

        Long schoolId = requestUser.getSchoolId();
        Long userId = requestUser.getId();
        // 查询分析计算导致的错误的数据源
        List<TBSyncTask> effectDatasource = tbSyncTaskService.list(new QueryWrapper<TBSyncTask>()
                .lambda()
                .eq(TBSyncTask::getSchoolId, schoolId)
                .eq(TBSyncTask::getType, PushTypeEnum.CALCULATE)
                .eq(TBSyncTask::getResult, TaskResultEnum.ERROR));

        List<String> finished = new ArrayList<>();
        finished.add(GradeAnalyzePaperStatusEnum.READY_TO_CALCULATE.name());
        finished.add(GradeAnalyzePaperStatusEnum.CALCULATING.name());
        finished.add(GradeAnalyzePaperStatusEnum.FINISH_CALCULATE.name());
        IPage<GradeInitResult> page = gradeInitializeMapper.findNeedAnalyzedPaper(new Page<>(pageNumber, pageSize), semesterId, examId, courseCode, userId, schoolId);
        for (GradeInitResult record : page.getRecords()) {
            String paperNumber = record.getPaperNumber();
            // 如果任意一个使用了该试卷配置的分析批次报试卷结构不一致地计算错误，则控制试卷结构配置页面显示更新试卷结构按钮
            record.setUseExamCloudStruct(effectDatasource.stream().anyMatch(e -> paperNumber.equals(e.getRemark())));
            String status = record.getStatus();
            if (SystemConstant.strNotNull(status)) {
                if (Arrays.stream(status.split(",")).anyMatch(finished::contains)) {
                    status = "已提交";
                } else {
                    status = "未提交";
                }
                record.setStatus(status);
            }
        }
        return page;
    }

    @Override
    public long countByPropositionTeacherId() {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = requestUser.getSchoolId();
        Long userId = requestUser.getId();
        List<GradeInitResult> findNeedAnalyzedPaper = gradeInitializeMapper.listNeedAnalyzedPaperToWork(schoolId, userId);
        return findNeedAnalyzedPaper.isEmpty() ? 0 : findNeedAnalyzedPaper.size();
    }

    @Override
    public IPage<GradeInitResult> listNeedAnalyzedPaperToWork(SysUser requestUser, Integer pageNumber, Integer pageSize) {
        Long schoolId = requestUser.getSchoolId();
        Long userId = requestUser.getId();
        return gradeInitializeMapper.listNeedAnalyzedPaperToWork(new Page<>(pageNumber, pageSize), schoolId, userId);
    }
}
